/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.schedule;

import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.constant.ApiPathConstants;
import com.zayac.admin.req.MemberDetailsReq;
import com.zayac.admin.req.MemberListReq;
import com.zayac.admin.req.PayRecordsListReq;
import com.zayac.admin.resp.*;
import com.zayac.admin.service.BannerService;
import com.zayac.admin.service.CompletableFutureWebClientService;
import com.zayac.admin.service.TelegramMessageService;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.AccountService;
import com.zayac.admin.system.service.UserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

/**
 * 定时任务 查询注册 新增用 暂定一分钟执行一次
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class MessageSchedule {

    private final UserService userService;
    private final AccountService accountService;
    private final CompletableFutureWebClientService completableFutureWebClientService;
    private final TelegramMessageService telegramMessageService;
    private final BannerService bannerService;

    //@Scheduled(fixedDelay = 60000)
    public void newCheckRegistrationAndNewDeposit() {
        // Get the current date and time
        LocalDate nowDate = LocalDate.now();
        LocalDateTime nowDateTime = LocalDateTime.now();
        // Fetch all enabled users
        List<UserDO> allEnableUser = userService.getAllEnabledUsers();

        allEnableUser.forEach(user -> processUser(user, nowDate, nowDateTime).join());
    }

    private CompletableFuture<Void> processUser(UserDO user, LocalDate nowDate, LocalDateTime nowDateTime) {
        List<AccountResp> accounts = accountService.getAccountsByUserId(user.getId(), DisEnableStatusEnum.ENABLE);
        List<CompletableFuture<Void>> futures = accounts.stream()
            .map(account -> processAccount(account, user, nowDate, nowDateTime))
            .toList();

        return CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]));
    }

    private CompletableFuture<Void> processAccount(AccountResp account,
                                                   UserDO user,
                                                   LocalDate nowDate,
                                                   LocalDateTime nowDateTime) {
        CompletableFuture<Banner> bannerFuture = bannerService.getLatestBannerInfoAsync(account);
        Banner prevBanner = bannerService.getPreviousBanner(account);

        return CompletableFuture.allOf(bannerFuture).thenCompose(v -> {
            Banner currentBanner = bannerFuture.join();

            CompletableFuture<Void> registrationProcess = processRegistration(account, user, currentBanner, prevBanner, nowDate);
            CompletableFuture<Void> depositProcess = processDeposits(account, user, currentBanner, prevBanner, nowDate, nowDateTime);

            return CompletableFuture.allOf(registrationProcess, depositProcess)
                .thenRun(() -> bannerService.updateBanner(account, currentBanner));
        });
    }

    private CompletableFuture<Void> processRegistration(AccountResp account,
                                                        UserDO currUser,
                                                        Banner banner,
                                                        Banner prevBanner,
                                                        LocalDate nowDate) {
        if (prevBanner != null && banner.getRegisterMembers() > prevBanner.getRegisterMembers()) {
            int registerCount = banner.getRegisterMembers() - prevBanner.getRegisterMembers();

            MemberListReq memberListReq = MemberListReq.builder()
                .registerStartDate(nowDate)
                .registerEndDate(nowDate)
                .startDate(nowDate)
                .endDate(nowDate)
                .registerSort(1)
                .pageSize(registerCount)
                .build();
            CompletableFuture<MemberPagination<List<Member>>> memberPaginationCompletableFuture = completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
                });
            return memberPaginationCompletableFuture.thenApply(MemberPagination::getList).thenAccept(members -> {
                if (!members.isEmpty()) {
                    String memberNames = members.stream().map(Member::getName).collect(Collectors.joining(", "));

                    String notification = String.format("👏 %s 注册: %d 用户: `%s` 总数:*%d*", account
                        .getNickname(), registerCount, memberNames, banner.getRegisterMembers());
                    if (DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
                        telegramMessageService.sendMessage(currUser.getBotToken(), currUser.getGroupId(), notification);
                    }
                }
            });
        }
        return CompletableFuture.completedFuture(null);
    }

    private CompletableFuture<Void> processDeposits(AccountResp account,
                                                    UserDO user,
                                                    Banner current,
                                                    Banner previous,
                                                    LocalDate nowDate,
                                                    LocalDateTime nowDateTime) {
        return CompletableFuture.runAsync(() -> {
            if (previous != null && current.getFirstDepositNum() > previous.getFirstDepositNum()) {
                int newDeposits = current.getFirstDepositNum() - previous.getFirstDepositNum();

                PayRecordsListReq req = PayRecordsListReq.builder()
                    .startDate(nowDate)
                    .endDate(nowDate)
                    .pageSize(100)
                    .payState(2)
                    .build();
                CompletableFuture<Pagination<List<PayRecord>>> paginationCompletableFuture = completableFutureWebClientService
                    .fetchDataForAccount(account, ApiPathConstants.PAY_RECORDS_LIST_URL, req, new ParameterizedTypeReference<>() {
                    });
                paginationCompletableFuture.thenApply(Pagination::getList).thenAccept(payRecords -> {
                    List<String> names = payRecords.stream()
                        .filter(record -> record.getCreatedAt().isAfter(nowDateTime.minusHours(1)))
                        .map(PayRecord::getName)
                        .distinct()
                        .toList();

                    names.forEach(name -> fetchMemberDetails(account, name, nowDate).thenAccept(member -> {
                        Optional<PayRecord> matchingRecord = payRecords.stream()
                            .filter(record -> record.getCreatedAt().equals(member.getFirstPayAt()))
                            .findFirst();
                        matchingRecord.ifPresent(record -> {
                            String depositResults = String.format("用户: `%s`, 首存金额: *%s*", member.getName(), record
                                .getScoreAmount());
                            String depResults = "🎉 " + account
                                .getNickname() + " 首存: " + newDeposits + depositResults + " 总数:*" + current
                                    .getFirstDepositNum() + "*";
                            if (DisEnableStatusEnum.ENABLE.equals(user.getNeedNotify())) {
                                telegramMessageService.sendMessage(user.getBotToken(), user.getGroupId(), depResults);
                            }
                        });
                    }).exceptionally(ex -> {
                        log.error("Error fetching details for member {}: {}", name, ex.getMessage());
                        return null;
                    }).join());
                }).exceptionally(ex -> {
                    log.error("Error processing deposits for account {}: {}", account.getId(), ex.getMessage());
                    return null;
                }).join();
            }
        });
    }

    private CompletableFuture<Member> fetchMemberDetails(AccountResp account, String name, LocalDate nowDate) {
        MemberListReq memberListReq = MemberListReq.builder()
            .name(name)
            .startDate(nowDate)
            .endDate(nowDate)
            .status(1)
            .build();

        CompletableFuture<MemberPagination<List<Member>>> memberFuture = completableFutureWebClientService
            .fetchDataForAccount(account, ApiPathConstants.MEMBER_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
            });

        return memberFuture.thenApply(MemberPagination::getList)
            .thenApply(list -> list.stream().findFirst())
            .thenCompose(optionalMember -> optionalMember.map(member -> fetchDetailedMemberInfo(account, member
                .getId(), nowDate)).orElseThrow(() -> new RuntimeException("没有找到匹配的成员信息")));
    }

    private CompletableFuture<Member> fetchDetailedMemberInfo(AccountResp account, Long memberId, LocalDate nowDate) {
        MemberDetailsReq detailsReq = MemberDetailsReq.builder()
            .id(memberId)
            .startDate(nowDate)
            .endDate(nowDate)
            .build();

        return completableFutureWebClientService
            .fetchDataForAccount(account, ApiPathConstants.MEMBER_DETAIL_URL, detailsReq, new ParameterizedTypeReference<>() {
            });
    }
}
