/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.schedule;

import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.req.team.TeamInfoReq;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.service.*;
import com.zayac.admin.system.model.entity.RoleDO;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.AccountService;
import com.zayac.admin.system.service.RoleService;
import com.zayac.admin.system.service.UserRoleService;
import com.zayac.admin.system.service.UserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Component
@RequiredArgsConstructor
public class TelegramTeamMessageSchedule {

    private final UserService userService;
    private final AccountService accountService;
    private final TeamService teamService;
    private final RoleService roleService;
    private final UserRoleService userRoleService;
    private final RegistrationService registrationService;
    private final DepositService depositService;

    private static final String MINISTER_ROLE_CODE = "minister";
    private static final long FIXED_DELAY = 60000L;

    @Scheduled(fixedDelay = FIXED_DELAY)
    public void CheckRegistrationAndNewDeposit() {
        LocalDate nowDate = LocalDate.now();
        LocalDateTime nowDateTime = LocalDateTime.now();
        RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
        List<Long> users = userRoleService.listUserIdByRoleId(minister.getId());
        users.forEach(userId -> {
            processUser(userService.getById(userId), nowDate, nowDateTime).join();
        });
    }

    private CompletableFuture<Void> processUser(UserDO minister, LocalDate nowDate, LocalDateTime nowDateTime) {
        List<AccountResp> accounts = accountService.getAccountsByUserId(minister.getId(), DisEnableStatusEnum.ENABLE)
                .stream()
                .filter(AccountResp::getIsTeam)
                .toList();

        List<CompletableFuture<Void>> futures = accounts.stream()
                .map(account -> processTeamAccount(minister, account, nowDate, nowDateTime))
                .toList();

        return CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]));
    }

    private CompletableFuture<Void> processTeamAccount(UserDO minister,
                                                       AccountResp account,
                                                       LocalDate nowDate,
                                                       LocalDateTime nowDateTime) {
        TeamInfoReq teamInfoReq = TeamInfoReq.builder()
                .startDate(nowDateTime.with(TemporalAdjusters.firstDayOfMonth()).with(LocalTime.MIN))
                .endDate(nowDateTime.with(TemporalAdjusters.lastDayOfMonth()).with(LocalTime.MAX))
                .build();
        CompletableFuture<Team> teamFuture = teamService.getLatestTeamInfoAsync(account, teamInfoReq);
        Team prevTeamInfo = teamService.getPreviousTeamInfo(account);

        return CompletableFuture.allOf(teamFuture).thenCompose(v -> {
            Team currentTeamInfo = teamFuture.join();
            log.info("Previous Team Info: {}", prevTeamInfo);
            log.info("Current Team Info: {}", currentTeamInfo);
            Map<String, TeamAccount> teamAccountMap = currentTeamInfo.getList()
                    .stream()
                    .collect(Collectors.toMap(TeamAccount::getAgentName, Function.identity()));

            CompletableFuture<Void> registrationProcess = registrationService
                    .processRegistration(minister, account, currentTeamInfo, prevTeamInfo, nowDate, teamAccountMap);
            CompletableFuture<Void> depositProcess = depositService
                    .processDeposits(minister, account, currentTeamInfo, prevTeamInfo, nowDate, nowDateTime);

            return CompletableFuture.allOf(registrationProcess, depositProcess)
                    .thenRun(() -> teamService.updateTeamInfo(account, currentTeamInfo));
        });
    }
}
