/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.constant.ApiPathConstants;
import com.zayac.admin.req.MemberDetailsReq;
import com.zayac.admin.req.PayRecordsListReq;
import com.zayac.admin.req.team.TeamMemberListReq;
import com.zayac.admin.resp.Member;
import com.zayac.admin.resp.MemberPagination;
import com.zayac.admin.resp.Pagination;
import com.zayac.admin.resp.PayRecord;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.resp.team.TeamAccountWithChange;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.AccountService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Service;
import top.continew.starter.core.exception.BusinessException;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.ArrayList;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class DepositService {
    private final CompletableFutureWebClientService completableFutureWebClientService;
    private final TelegramMessageService telegramMessageService;
    private final AccountService accountService;

    public CompletableFuture<Void> processDeposits(UserDO minister,
                                                   AccountResp account,
                                                   Team currentTeam,
                                                   Team previousTeam,
                                                   LocalDate nowDate,
                                                   LocalDateTime nowDateTime) {
        return CompletableFuture.runAsync(() -> {
            if (previousTeam != null && currentTeam.getFirstDepositNum() > previousTeam.getFirstDepositNum()) {
                PayRecordsListReq req = PayRecordsListReq.builder()
                        .startDate(nowDate)
                        .endDate(nowDate)
                        .pageSize(100)
                        .payState(2)
                        .build();
                CompletableFuture<Pagination<List<PayRecord>>> paginationCompletableFuture = completableFutureWebClientService
                        .fetchDataForAccount(account, ApiPathConstants.PAY_RECORDS_LIST_URL, req, new ParameterizedTypeReference<>() {
                        });
                List<TeamAccountWithChange> changedTeamAccounts = findChangedTeamAccount(previousTeam, currentTeam);
                Set<String> changedAgentNames = changedTeamAccounts.stream()
                        .filter(teamAccountWithChange -> teamAccountWithChange.getNewDepositNum() > 0)
                        .map(TeamAccountWithChange::getAgentName)
                        .collect(Collectors.toSet());
                paginationCompletableFuture.thenApply(Pagination::getList).thenAccept(payRecords -> {
                    Map<String, List<String>> agentNameWithNames = payRecords.stream()
                            .filter(record -> record.getCreatedAt().isAfter(nowDateTime.minusHours(1)) && changedAgentNames
                                    .contains(record.getAgentName()))
                            .collect(Collectors.groupingBy(PayRecord::getAgentName, Collectors
                                    .mapping(PayRecord::getName, Collectors.collectingAndThen(Collectors
                                            .toSet(), ArrayList::new))));

                    agentNameWithNames.forEach((agentName, names) -> {
                        StringBuilder depositResults = new StringBuilder();
                        AtomicInteger depositCounter = new AtomicInteger(0);

                        TeamAccountWithChange targetTeamAccount = changedTeamAccounts.stream()
                                .filter(teamAccount -> StrUtil.equals(teamAccount.getAgentName(), agentName))
                                .findFirst()
                                .orElseThrow(() -> new BusinessException(String
                                        .format("can not find agent name %s", agentName)));

                        List<CompletableFuture<Void>> fetchFutures = names.stream()
                                .map(name -> fetchMemberDetails(account, name, nowDate).thenAccept(member -> {
                                    payRecords.stream()
                                            .filter(record -> record.getCreatedAt().equals(member.getFirstPayAt()))
                                            .findFirst()
                                            .ifPresent(record -> {
                                                if (depositCounter.getAndIncrement() < targetTeamAccount.getNewDepositNum()) {
                                                    depositResults.append(telegramMessageService
                                                            .buildDepositResultsMessage(member.getName(), record.getScoreAmount()));
                                                }
                                            });
                                }).exceptionally(ex -> {
                                    log.error("Error fetching details for member {}: {}", name, ex.getMessage());
                                    return null;
                                }))
                                .toList();

                        CompletableFuture<Void> allFetches = CompletableFuture.allOf(fetchFutures
                                .toArray(new CompletableFuture[0]));

                        allFetches.thenRun(() -> {
                            if (!depositResults.isEmpty()) {
                                String notification = telegramMessageService
                                        .buildDepositMessage(agentName, targetTeamAccount.getNewDepositNum(), depositResults
                                                .toString(), targetTeamAccount.getFirstDepositNum());
                                UserDO currUser = accountService.getUserByAccountUsername(agentName);
                                if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
                                    String botToken = StrUtil.isEmpty(currUser.getBotToken()) ? minister.getBotToken() : currUser.getBotToken();
                                    telegramMessageService.sendMessage(botToken, currUser.getGroupId(), notification);
                                }
                                telegramMessageService
                                        .sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, notification);
                            }
                        }).exceptionally(ex -> {
                            log.error("Error sending notification for account {}: {}", account.getId(), ex
                                    .getMessage());
                            return null;
                        });
                    });
                }).exceptionally(ex -> {
                    log.error("Error processing deposits for account {}: {}", account.getId(), ex.getMessage());
                    return null;
                }).join();
            }
        });
    }

    private CompletableFuture<Member> fetchMemberDetails(AccountResp account, String name, LocalDate nowDate) {
        TeamMemberListReq memberListReq = TeamMemberListReq.builder()
                .name(name)
                .startDate(nowDate)
                .endDate(nowDate)
                .status(1)
                .build();

        CompletableFuture<MemberPagination<List<Member>>> memberFuture = completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_TEAM_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
                });

        return memberFuture.thenApply(MemberPagination::getList)
                .thenApply(list -> list.stream().findFirst())
                .thenCompose(optionalMember -> optionalMember.map(member -> fetchDetailedMemberInfo(account, member
                        .getId(), nowDate)).orElseThrow(() -> new RuntimeException("没有找到匹配的成员信息")));
    }

    private CompletableFuture<Member> fetchDetailedMemberInfo(AccountResp account, Long memberId, LocalDate nowDate) {
        MemberDetailsReq detailsReq = MemberDetailsReq.builder()
                .id(memberId)
                .startDate(nowDate)
                .endDate(nowDate)
                .build();

        return completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_DETAIL_URL, detailsReq, new ParameterizedTypeReference<>() {
                });
    }

    private List<TeamAccountWithChange> findChangedTeamAccount(Team prevTeam, Team currTeam) {
        Map<Long, TeamAccount> team2AccountMap = currTeam.getList()
                .stream()
                .collect(Collectors.toMap(TeamAccount::getId, account -> account));

        return prevTeam.getList()
                .stream()
                .filter(account1 -> team2AccountMap.containsKey(account1.getId()))
                .map(account1 -> {
                    TeamAccount account2 = team2AccountMap.get(account1.getId());
                    TeamAccountWithChange changedAccount = new TeamAccountWithChange();
                    BeanUtil.copyProperties(account2, changedAccount);
                    if (account1.getFirstDepositNum() != account2.getFirstDepositNum()) {
                        changedAccount.setNewDepositNum(account2.getFirstDepositNum() - account1.getFirstDepositNum());
                    }
                    if (account1.getSubMemberNum() != account2.getSubMemberNum()) {
                        changedAccount.setNewRegisterNum(account2.getSubMemberNum() - account1.getSubMemberNum());
                    }
                    return changedAccount;
                })
                .collect(Collectors.toList());
    }
}